@@ -129,7 +129,7 @@ formatting = {
             'detailedWhereAmI' : 'pageSummary'
             },
         pyatspi.ROLE_ALERT: {
-            'unfocused': 'roleName + labelOrName + pause + (expandedEOCs or unrelatedLabels or description)'
+            'unfocused': 'roleName + labelOrName + pause + alertText'
             },
         pyatspi.ROLE_ANIMATION: {
             'unfocused': 'labelAndName'
@@ -305,8 +305,8 @@ formatting = {
             },
         pyatspi.ROLE_LIST_ITEM: {
             'focused': 'expandableState',
-            'unfocused': '(labelOrName or (displayedText + allTextSelection)) + checkedStateIfCheckable + pause + expandableState + pause + positionInList + pause + listBoxItemWidgets',
-            'basicWhereAmI': 'label + roleName + pause + (name or displayedText) + checkedStateIfCheckable + pause + positionInList + pause + expandableState + (nodeLevel or nestingLevel) + pause'
+            'unfocused': '(labelOrName or (displayedText + allTextSelection)) + checkedStateIfCheckable + pause + unselectedStateIfSelectable + pause + expandableState + pause + positionInList + pause + listBoxItemWidgets',
+            'basicWhereAmI': 'label + roleName + pause + (name or displayedText) + checkedStateIfCheckable + pause + unselectedStateIfSelectable + pause + positionInList + pause + expandableState + (nodeLevel or nestingLevel) + pause'
             },
         pyatspi.ROLE_MATH: {
             'unfocused': 'math',
@@ -411,6 +411,10 @@ formatting = {
             'unfocused': 'labelOrName + radioState + roleName + availability + ' + MNEMONIC + ' + accelerator + positionInList',
             'basicWhereAmI': 'ancestors + labelOrName + roleName + radioState + accelerator + positionInList + ' + MNEMONIC
             },
+        'ROLE_REGION': {
+            'focused': 'leaving or (roleName + labelOrName)',
+            'unfocused': 'labelOrName + roleName + currentLineText + allTextSelection'
+            },
         pyatspi.ROLE_ROW_HEADER: {
             'focused': 'labelAndName + roleName + pause + sortOrder',
             'unfocused': '((substring and currentLineText) or labelAndName) + roleName + pause + sortOrder'
@@ -519,6 +523,10 @@ formatting = {
             'unfocused': 'roleName + labelAndName',
             'basicWhereAmI': 'roleName + labelAndName'
             },
+        pyatspi.ROLE_TREE: {
+            'focused': 'labelAndName + roleName',
+            'unfocused': 'labelAndName + roleName',
+            },
         pyatspi.ROLE_TREE_ITEM: {
             'focused': 'expandableState',
             'unfocused': '(labelOrName or displayedText) + pause + expandableState + pause + positionInList',
@@ -821,10 +829,13 @@ formatting = {
         pyatspi.ROLE_TOOL_BAR: {
             'unfocused': '[Component(obj, asString(labelOrName + roleName))]',
             },
+        pyatspi.ROLE_TREE: {
+            'unfocused': '[Component(obj, asString(labelOrName + roleName))]',
+            },
         pyatspi.ROLE_TREE_ITEM: {
             'unfocused': '((substring and ' + BRAILLE_TEXT + ')\
                           or ([Component(obj, asString(labelOrName + expandableState))]\
-                              + (nestingLevel and [Region(" " + asString(nestingLevel))])))',
+                              + (nodeLevel and [Region(" " + asString(nodeLevel))])))',
             },
         #pyatspi.ROLE_TREE: 'default'
         #pyatspi.ROLE_TREE_TABLE: 'default'
