@@ -78,7 +78,7 @@ class Generator:
         self._activeProgressBars = {}
         self._methodsDict = {}
         for method in \
-            [z for z in [getattr(self, y).__get__(self, self.__class__) for y in [x for x in dir(self) if x.startswith(METHOD_PREFIX)]] if isinstance(z, collections.Callable)]:
+            [z for z in [getattr(self, y).__get__(self, self.__class__) for y in [x for x in dir(self) if x.startswith(METHOD_PREFIX)]] if isinstance(z, collections.abc.Callable)]:
             name = method.__name__[len(METHOD_PREFIX):]
             name = name[0].lower() + name[1:]
             self._methodsDict[name] = method
@@ -342,8 +342,8 @@ class Generator:
                 elif obj.parent and obj.parent.getRole() == pyatspi.ROLE_LINK:
                     link = obj.parent
                 if link:
-                    basename = self._script.utilities.linkBasename(link)
-                    if basename and basename.isalpha():
+                    basename = self._script.utilities.linkBasenameToName(link)
+                    if basename:
                         result.append(basename)
         # To make the unlabeled icons in gnome-panel more accessible.
         try:
@@ -401,11 +401,10 @@ class Generator:
         If the label cannot be found, the name will be used instead.
         If the name cannot be found, an empty array will be returned.
         """
-        result = []
-        result.extend(self._generateLabel(obj, **args))
+        result = self._generateLabel(obj, **args)
         if not result:
-            if obj.name and (len(obj.name)):
-                result.append(obj.name)
+            result = self._generateName(obj, **args)
+
         return result
 
     def _generateDescription(self, obj, **args):
@@ -921,9 +920,17 @@ class Generator:
             return []
 
         rows, cols = self._script.utilities.rowAndColumnCount(obj)
+
+        # This suggests broken or missing table interface.
         if rows < 0 or cols < 0:
             return []
 
+        # This can happen if an author uses ARIA incorrectly, e.g. a grid whose
+        # immediate child is a gridcell rather than a row. In that case, just
+        # announce the role name.
+        if rows == 0 and cols == 0:
+            return self._generateRoleName(obj, **args)
+
         return [messages.tableSize(rows, cols)]
 
     def _generateTableCellRow(self, obj, **args):
@@ -1281,6 +1288,8 @@ class Generator:
         if self._script.utilities.isContentSuggestion(obj):
             return 'ROLE_CONTENT_SUGGESTION'
         if self._script.utilities.isLandmark(obj):
+            if self._script.utilities.isLandmarkRegion(obj):
+                return 'ROLE_REGION'
             return pyatspi.ROLE_LANDMARK
         if self._script.utilities.isFocusableLabel(obj):
             return pyatspi.ROLE_LIST_ITEM
